# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimDate,
    HprimForeign,
)

logger = logging.getLogger("driverng.fanci")


class FanciSegmentMSH(HeaderSegment):
    sending_application = HprimString(3)
    sending_facility = HprimString(4)
    receiving_facility = HprimString(6)
    date = HprimDate(7)
    message_type = HprimString(9)
    message_control_id = HprimString(10)
    processing_id = HprimString(11, default="P")
    version_id = HprimString(12, default="HL7 version")
    accept_ack_type = HprimString(15, default="AL")
    application_ack_type = HprimString(16)

    patients = HprimForeign("PID", "header")
    orders = HprimForeign("OBR", "header")


class FanciSegmentPID(Segment):
    patient_id = HprimString(4)
    patient_name = HprimString(6)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    visits = HprimForeign("PV1", "patient")
    orders = HprimForeign("OBR", "patient")
    common_order = HprimForeign("ORC", "patient")


class FanciSegmentPV1(Segment):
    referring_doctor = HprimString(8)
    visit_number = HprimString(19)

    orders = HprimForeign("OBR", "patient")


class FanciSegmentORC(Segment):
    order_control = HprimString(1, default="NW")
    order_status = HprimString(5, default="SC")
    date = HprimDate(9)


class FanciSegmentOBR(Segment):
    universal_test_id = HprimString((5, 4))
    test_label = HprimString((5, 5))
    requested_date = HprimDate(6)
    result_status = HprimString(25, default="F")

    results = HprimForeign("OBX", "order")
    common_order = HprimForeign("ORC", "order")


class FanciSegmentOBX(Segment):
    value_type = HprimString(2, default="NM")
    observation_identifier = HprimString(3)
    test_code = HprimString((4, 4))
    test_label = HprimString((4, 5))
    test_flag = HprimString((4, 6))
    observation_value = HprimString(6)
    units = HprimString(7)
    result_status = HprimString(11)
    date = HprimDate(15)


class FanciSegmentQRD(Segment):
    format_code = HprimString(2)
    priority = HprimString(3)
    quantity_limited_request = HprimString(7)
    who_subject_filter = HprimString(8)
    what_subject_filter = HprimString(9)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "fanci",
        "label": "FANci",
    }
    DEFAULT_SEPARATORS = "|^~\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    SEGMENT_CLASS_MAP = {
        "MSH": FanciSegmentMSH,
        "PID": FanciSegmentPID,
        "PV1": FanciSegmentPV1,
        "ORC": FanciSegmentORC,
        "OBR": FanciSegmentOBR,
        "OBX": FanciSegmentOBX,
        "QRD": FanciSegmentQRD,
    }

    def create(self, **kwargs):
        return FanciSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            for sample, action in patient.samples:
                seg_pid = msg.patients.add(
                    patient_id=sample.sample_id,
                    patient_name="{} {}".format(
                        patient.last_name,
                        patient.first_name
                    ),
                    birthdate=patient.birthdate,
                    sex=patient.sex
                )
                for order in sample["orders"]:
                    seg_pid.orders.add(
                        universal_test_id=order.code,
                        requested_date=sample.request_date
                    )
        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_obr in seg_pid.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_pid.patient_id,
                    type="patient",
                )
                for seg_obx in seg_obr.results:
                    test = "{}_{}".format(
                        seg_obr.universal_test_id,
                        seg_obx.test_code,
                    )
                    sample.results.add(
                        code=test,
                        value=seg_obx.observation_value,
                        date=seg_obx.date,
                    )
        return worklist
