# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)
from driverng.utils import num_years


class KiestraSegmentH(HeaderSegment):
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KiestraSegmentP(Segment):
    patient_id = HprimString(4)
    age = HprimString(8)
    sex = HprimString(9)
    special_field1 = HprimString(15)
    special_field2 = HprimString(16)
    active_medication = HprimString(20)
    practice_field1 = HprimString(22)
    practice_field2 = HprimString(23)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class KiestraSegmentO(Segment):
    accession_number = HprimString((3, 1))
    accession_no_ref = HprimString((3, 2))
    test_id = HprimString((5, 4))
    specimen_type = HprimString((16, 1))
    users_field_no1a = HprimString((20, 1))
    users_field_no1b = HprimString((20, 2))
    users_field_no2a = HprimString((21, 1))
    users_field_no2b = HprimString((21, 2))

    results = HprimForeign("R", "order")


class KiestraSegmentR(Segment):
    result_type_code = HprimString((3, 4))
    consumable_sequence_number = HprimString((3, 5))
    # No d'isolat
    carrier_seq_no = HprimString((3, 6))
    carrier_comment = HprimString((3, 7))
    test_id = HprimString((4, 1))
    result_value1 = HprimString((4, 2))
    result_value2 = HprimString((4, 3))
    result_value3 = HprimString((4, 4))
    audit = HprimString(11)
    datetime_test = HprimDate(13)
    instrument_id = HprimString(14)


class KiestraSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)


class KiestraSegmentQ(Segment):
    accession_number = HprimString((3, 2))


class KiestraSegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "kiestra",
        "label": "Kiestra BD"
    }
    SEGMENT_CLASS_MAP = {
        "H": KiestraSegmentH,
        "P": KiestraSegmentP,
        "O": KiestraSegmentO,
        "R": KiestraSegmentR,
        "C": KiestraSegmentC,
        "Q": KiestraSegmentQ,
        "L": KiestraSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = KiestraSegmentH(type_segment="H", **kwargs)
        message.end = KiestraSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            age = ""
            age = num_years(patient["birthdate"], patient["collection_date"])
            if age is None:
                age = ""
            seg_p = msg.patients.add(
                patient_id=patient.patient_id
            )
            for sample, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    accession_number=sample.sample_id
                )
                list_germes = []
                for result in sample.results:
                    if "germes" in result["resu_json"]:
                        for germe in result["resu_json"]["germes"]:
                            if "id" in germe:
                                list_germes.append(germe.get("id"))
                            elif "Id" in germe:
                                list_germes.append(germe.get("Id"))
                    if list_germes:
                        for germe in list_germes:
                            seg_o.results.add(
                                result_type_code="ORG",
                                # Name of organism
                                test_id=germe,
                                # The amount of growth (Quantity)
                                result_value1=result.value,
                            )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.accession_number,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                list_barcode_media = []
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.accession_number,
                    )
                    if seg_o.users_field_no2b:
                        list_barcode_media.append(seg_o.users_field_no2b)
                        sample.barcode_media = list_barcode_media
                    specimen_code = ""
                    code_identification = ""
                    liste_identification_germes = []
                    liste_atb = []
                    code_germe = ""
                    for seg_r in seg_o.results:
                        if seg_r.result_type_code not in ('ORG', 'IT', 'RES', 'AB'):
                            specimen_code = seg_r.result_type_code
                        elif seg_r.result_type_code == "ORG":
                            sample.results.add(
                                code="{}ID{}".format(specimen_code, seg_r.carrier_seq_no),
                                value=seg_r.test_id
                            )
                            code_identification = "IDENT_{}{}".format(specimen_code, seg_r.carrier_seq_no)
                            liste_identification_germes.append({
                                "rang": seg_r.carrier_seq_no,
                                "id": seg_r.test_id,
                            })
                            code_germe = "G_{}".format(seg_r.test_id)
                            if seg_r.result_value1:
                                sample.results.add(
                                    code="{}QT{}".format(specimen_code, seg_r.carrier_seq_no),
                                    value=seg_r.result_value1
                                )
                        elif seg_r.result_type_code == "IT":
                            sample.results.add(
                                code="{}{}{}".format(specimen_code, seg_r.test_id, seg_r.carrier_seq_no),
                                value=seg_r.result_value1
                            )
                        elif seg_r.result_type_code == "AB":
                            code_antibio = "A_{}".format(seg_r.test_id)
                            liste_atb.append((
                                ("ANTIBIO", code_antibio),
                                ("CMI", seg_r.result_value1),
                                ("SENSI", seg_r.result_value3)
                            ))
                    if code_identification and liste_identification_germes:
                        sample.results.add(
                            code=code_identification,
                            value={
                                "seq": "0",
                                "type_resultat": "identification_germes",
                                "germes": liste_identification_germes
                            },
                            instrument=seg_r.instrument_id
                        )
                    if liste_atb:
                        sample.results.add(
                            code=specimen_code,
                            value=code_germe,
                            atb=liste_atb,
                            date=seg_r.datetime_test,
                            instrument=seg_r.instrument_id
                        )
        return worklist
